@@ -48,8 +48,9 @@ pub fn format_location(frame: &JsStackFrame) -> String {
     return cyan("native").to_string();
   }
   let mut result = String::new();
-  if let Some(file_name) = &frame.file_name {
-    result += &cyan(&format_file_name(file_name)).to_string();
+  let file_name = frame.file_name.clone().unwrap_or_default();
+  if !file_name.is_empty() {
+    result += &cyan(&format_file_name(&file_name)).to_string();
   } else {
     if frame.is_eval {
       result +=
@@ -128,55 +129,15 @@ fn format_frame(frame: &JsStackFrame) -> String {
   result
 }
 
-#[allow(clippy::too_many_arguments)]
-fn format_stack(
-  is_error: bool,
-  message_line: &str,
-  cause: Option<&str>,
-  source_line: Option<&str>,
-  start_column: Option<i64>,
-  end_column: Option<i64>,
-  frames: &[JsStackFrame],
-  level: usize,
-) -> String {
-  let mut s = String::new();
-  s.push_str(&format!("{:indent$}{}", "", message_line, indent = level));
-  s.push_str(&format_maybe_source_line(
-    source_line,
-    start_column,
-    end_column,
-    is_error,
-    level,
-  ));
-  for frame in frames {
-    s.push_str(&format!(
-      "\n{:indent$}    at {}",
-      "",
-      format_frame(frame),
-      indent = level
-    ));
-  }
-  if let Some(cause) = cause {
-    s.push_str(&format!(
-      "\n{:indent$}Caused by: {}",
-      "",
-      cause,
-      indent = level
-    ));
-  }
-  s
-}
-
 /// Take an optional source line and associated information to format it into
 /// a pretty printed version of that line.
 fn format_maybe_source_line(
   source_line: Option<&str>,
-  start_column: Option<i64>,
-  end_column: Option<i64>,
+  column_number: Option<i64>,
   is_error: bool,
   level: usize,
 ) -> String {
-  if source_line.is_none() || start_column.is_none() || end_column.is_none() {
+  if source_line.is_none() || column_number.is_none() {
     return "".to_string();
   }
 
@@ -191,37 +152,24 @@ fn format_maybe_source_line(
     return format!("\n{}", source_line);
   }
 
-  assert!(start_column.is_some());
-  assert!(end_column.is_some());
   let mut s = String::new();
-  let start_column = start_column.unwrap();
-  let end_column = end_column.unwrap();
+  let column_number = column_number.unwrap();
 
-  if start_column as usize >= source_line.len() {
+  if column_number as usize > source_line.len() {
     return format!(
       "\n{} Couldn't format source line: Column {} is out of bounds (source may have changed at runtime)",
-      crate::colors::yellow("Warning"), start_column + 1,
+      crate::colors::yellow("Warning"), column_number,
     );
   }
 
-  // TypeScript uses `~` always, but V8 would utilise `^` always, even when
-  // doing ranges, so here, if we only have one marker (very common with V8
-  // errors) we will use `^` instead.
-  let underline_char = if (end_column - start_column) <= 1 {
-    '^'
-  } else {
-    '~'
-  };
-  for _i in 0..start_column {
+  for _i in 0..(column_number - 1) {
     if source_line.chars().nth(_i as usize).unwrap() == '\t' {
       s.push('\t');
     } else {
       s.push(' ');
     }
   }
-  for _i in 0..(end_column - start_column) {
-    s.push(underline_char);
-  }
+  s.push('^');
   let color_underline = if is_error {
     red(&s).to_string()
   } else {
@@ -233,6 +181,43 @@ fn format_maybe_source_line(
   format!("\n{}{}\n{}{}", indent, source_line, indent, color_underline)
 }
 
+fn format_js_error(js_error: &JsError, is_child: bool) -> String {
+  let mut s = String::new();
+  s.push_str(&js_error.exception_message);
+  if let Some(aggregated) = &js_error.aggregated {
+    for aggregated_error in aggregated {
+      let error_string = format_js_error(aggregated_error, true);
+      for line in error_string.trim_start_matches("Uncaught ").lines() {
+        s.push_str(&format!("\n    {}", line));
+      }
+    }
+  }
+  let column_number = js_error
+    .source_line_frame_index
+    .and_then(|i| js_error.frames.get(i).unwrap().column_number);
+  s.push_str(&format_maybe_source_line(
+    if is_child {
+      None
+    } else {
+      js_error.source_line.as_deref()
+    },
+    column_number,
+    true,
+    0,
+  ));
+  for frame in &js_error.frames {
+    s.push_str(&format!("\n    at {}", format_frame(frame)));
+  }
+  if let Some(cause) = &js_error.cause {
+    let error_string = format_js_error(cause, true);
+    s.push_str(&format!(
+      "\nCaused by: {}",
+      error_string.trim_start_matches("Uncaught ")
+    ));
+  }
+  s
+}
+
 /// Wrapper around deno_core::JsError which provides colorful
 /// string representation.
 #[derive(Debug)]
@@ -254,45 +239,7 @@ impl Deref for PrettyJsError {
 
 impl fmt::Display for PrettyJsError {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-    let mut frames = self.0.frames.clone();
-
-    // When the stack frame array is empty, but the source location given by
-    // (script_resource_name, line_number, start_column + 1) exists, this is
-    // likely a syntax error. For the sake of formatting we treat it like it was
-    // given as a single stack frame.
-    if frames.is_empty()
-      && self.0.script_resource_name.is_some()
-      && self.0.line_number.is_some()
-      && self.0.start_column.is_some()
-    {
-      frames = vec![JsStackFrame::from_location(
-        self.0.script_resource_name.clone(),
-        self.0.line_number,
-        self.0.start_column.map(|n| n + 1),
-      )];
-    }
-
-    let cause = self
-      .0
-      .cause
-      .clone()
-      .map(|cause| format!("{}", PrettyJsError(*cause)));
-
-    write!(
-      f,
-      "{}",
-      &format_stack(
-        true,
-        &self.0.exception_message,
-        cause.as_deref(),
-        self.0.source_line.as_deref(),
-        self.0.start_column,
-        self.0.end_column,
-        &frames,
-        0
-      )
-    )?;
-    Ok(())
+    write!(f, "{}", &format_js_error(&self.0, false))
   }
 }
 
@@ -305,22 +252,17 @@ mod tests {
 
   #[test]
   fn test_format_none_source_line() {
-    let actual = format_maybe_source_line(None, None, None, false, 0);
+    let actual = format_maybe_source_line(None, None, false, 0);
     assert_eq!(actual, "");
   }
 
   #[test]
   fn test_format_some_source_line() {
-    let actual = format_maybe_source_line(
-      Some("console.log('foo');"),
-      Some(8),
-      Some(11),
-      true,
-      0,
-    );
+    let actual =
+      format_maybe_source_line(Some("console.log('foo');"), Some(9), true, 0);
     assert_eq!(
       strip_ansi_codes(&actual),
-      "\nconsole.log(\'foo\');\n        ~~~"
+      "\nconsole.log(\'foo\');\n        ^"
     );
   }
 }
