@@ -15,7 +15,7 @@ use std::pin::Pin;
 use std::rc::Rc;
 use std::str;
 
-pub(crate) struct CliModuleLoader {
+pub struct CliModuleLoader {
   pub lib: TypeLib,
   /// The initial set of permissions used to resolve the static imports in the
   /// worker. They are decoupled from the worker (dynamic) permissions since
@@ -108,7 +108,7 @@ impl ModuleLoader for CliModuleLoader {
 
     async move {
       ps.prepare_module_load(
-        vec![(specifier, deno_graph::ModuleKind::Esm)],
+        vec![specifier],
         is_dynamic,
         lib,
         root_permissions,
